@@ -79,6 +79,12 @@ other actions from the bisect popup (\
   (interactive (if (magit-bisect-in-progress-p)
                    (user-error "Already bisecting")
                  (magit-bisect-start-read-args)))
+  (unless (magit-rev-ancestor-p good bad)
+    (user-error
+     "The good revision (%s) has to be an ancestor of the bad one (%s)"
+     good bad))
+  (when (magit-anything-modified-p)
+    (user-error "Cannot bisect with uncommitted changes"))
   (magit-git-bisect "start" (list bad good) t))
 
 (defun magit-bisect-start-read-args ()
@@ -206,5 +212,6 @@ bisect run'."
         (magit-insert-section (bisect-item)
           (insert hash " is the first bad commit\n"))))))
 
+;;; _
 (provide 'magit-bisect)
 ;;; magit-bisect.el ends here
