@@ -35,6 +35,9 @@
 (require 'cl-lib)
 (require 'dash)
 
+(eval-when-compile
+  (require 'subr-x))
+
 (require 'with-editor)
 (require 'magit-utils)
 (require 'magit-section)
@@ -109,7 +112,7 @@ displays the text of `magit-process-error-summary' instead."
                  integer))
 
 (defcustom magit-credential-cache-daemon-socket
-  (--some (-let [(prog . args) (split-string it)]
+  (--some (pcase-let ((`(,prog . ,args) (split-string it)))
             (if (and prog
                      (string-match-p
                       "\\`\\(?:\\(?:/.*/\\)?git-credential-\\)?cache\\'" prog))
@@ -155,8 +158,8 @@ itself from the hook, to avoid further futile attempts."
 
 (defcustom magit-process-password-prompt-regexps
   '("^\\(Enter \\)?[Pp]assphrase\\( for \\(RSA \\)?key '.*'\\)?: ?$"
-    ;; match-group 99 is used to identify a host
-    "^\\(Enter \\)?[Pp]assword\\( for '\\(?99:.*\\)'\\)?: ?$"
+    ;; Match-group 99 is used to identify the "user@host" part.
+    "^\\(Enter \\)?[Pp]assword\\( for '\\(https?://\\)?\\(?99:.*\\)'\\)?: ?$"
     "^.*'s password: ?$"
     "^Yubikey for .*: ?$"
     "^Enter PIN for .*: ?$")
@@ -282,12 +285,11 @@ optional NODISPLAY is non-nil also display it."
 (defun magit-process-kill ()
   "Kill the process at point."
   (interactive)
-  (magit-section-when process
-    (let ((process (oref it value)))
-      (unless (eq (process-status process) 'run)
-        (user-error "Process isn't running"))
-      (magit-confirm 'kill-process)
-      (kill-process process))))
+  (when-let ((process (magit-section-value-if 'process)))
+    (unless (eq (process-status process) 'run)
+      (user-error "Process isn't running"))
+    (magit-confirm 'kill-process)
+    (kill-process process)))
 
 ;;; Synchronous Processes
 
@@ -347,20 +349,21 @@ Process output goes into a new section in the buffer returned by
   "Call PROGRAM synchronously in a separate process.
 Process output goes into a new section in the buffer returned by
 `magit-process-buffer'."
-  (-let [(process-buf . section) (magit-process-setup program args)]
+  (pcase-let ((`(,process-buf . ,section)
+               (magit-process-setup program args)))
     (magit-process-finish
      (let ((inhibit-read-only t))
        (apply #'magit-process-file program nil process-buf nil args))
      process-buf (current-buffer) default-directory section)))
 
-(defun magit-process-file (&rest args)
+(defun magit-process-file (process &optional infile buffer display &rest args)
   "Process files synchronously in a separate process.
 Identical to `process-file' but temporarily enable Cygwin's
 \"noglob\" option during the call and ensure unix eol
 conversion."
   (let ((process-environment (magit-process-environment))
         (default-process-coding-system (magit--process-coding-system)))
-    (apply #'process-file args)))
+    (apply #'process-file process infile buffer display args)))
 
 (defun magit-process-environment ()
   ;; The various w32 hacks are only applicable when running on the
@@ -393,7 +396,6 @@ Option `magit-git-executable' specifies the Git executable and
 option `magit-git-global-arguments' specifies constant arguments.
 The remaining arguments ARGS specify arguments to Git, they are
 flattened before use."
-  (declare (indent 1))
   (when (eq system-type 'windows-nt)
     ;; On w32, git expects UTF-8 encoded input, ignore any user
     ;; configuration telling us otherwise (see #3250).
@@ -407,12 +409,12 @@ flattened before use."
                     (eq (process-status magit-this-process) 'run))
           (sleep-for 0.005)))
     (run-hooks 'magit-pre-call-git-hook)
-    (-let* ((process-environment (magit-process-environment))
-            (default-process-coding-system (magit--process-coding-system))
-            (flat-args (magit-process-git-arguments args))
-            ((process-buf . section)
-             (magit-process-setup magit-git-executable flat-args))
-            (inhibit-read-only t))
+    (pcase-let* ((process-environment (magit-process-environment))
+                 (default-process-coding-system (magit--process-coding-system))
+                 (flat-args (magit-process-git-arguments args))
+                 (`(,process-buf . ,section)
+                  (magit-process-setup magit-git-executable flat-args))
+                 (inhibit-read-only t))
       (magit-process-finish
        (apply #'call-process-region (point-min) (point-max)
               magit-git-executable nil process-buf nil flat-args)
@@ -519,18 +521,19 @@ After the process returns, `magit-process-sentinel' refreshes the
 buffer that was current when `magit-start-process' was called (if
 it is a Magit buffer and still alive), as well as the respective
 Magit status buffer."
-  (-let* (((process-buf . section)
-           (magit-process-setup program args))
-          (process
-           (let ((process-connection-type
-                  ;; Don't use a pty, because it would set icrnl
-                  ;; which would modify the input (issue #20).
-                  (and (not input) magit-process-connection-type))
-                 (process-environment (magit-process-environment))
-                 (default-process-coding-system (magit--process-coding-system)))
-             (apply #'start-file-process
-                    (file-name-nondirectory program)
-                    process-buf program args))))
+  (pcase-let*
+      ((`(,process-buf . ,section)
+        (magit-process-setup program args))
+       (process
+        (let ((process-connection-type
+               ;; Don't use a pty, because it would set icrnl
+               ;; which would modify the input (issue #20).
+               (and (not input) magit-process-connection-type))
+              (process-environment (magit-process-environment))
+              (default-process-coding-system (magit--process-coding-system)))
+          (apply #'start-file-process
+                 (file-name-nondirectory program)
+                 process-buf program args))))
     (with-editor-set-process-filter process #'magit-process-filter)
     (set-process-sentinel process #'magit-process-sentinel)
     (set-process-buffer   process process-buf)
@@ -600,7 +603,8 @@ Magit status buffer."
       (cond
        ((and args (equal program magit-git-executable))
         (setq args (-split-at (length magit-git-global-arguments) args))
-        (insert (propertize program 'face 'magit-section-heading) " ")
+        (insert (propertize (file-name-nondirectory program)
+                            'face 'magit-section-heading) " ")
         (insert (propertize (char-to-string magit-ellipsis)
                             'face 'magit-section-heading
                             'help-echo (mapconcat #'identity (car args) " ")))
@@ -610,7 +614,8 @@ Magit status buffer."
        ((and args (equal program shell-file-name))
         (insert (propertize (cadr args) 'face 'magit-section-heading)))
        (t
-        (insert (propertize program 'face 'magit-section-heading) " ")
+        (insert (propertize (file-name-nondirectory program)
+                            'face 'magit-section-heading) " ")
         (insert (propertize (mapconcat #'shell-quote-argument args " ")
                             'face 'magit-section-heading))))
       (magit-insert-heading)
@@ -662,10 +667,10 @@ Magit status buffer."
   "Special sentinel used by `magit-run-git-sequencer'."
   (when (memq (process-status process) '(exit signal))
     (magit-process-sentinel process event)
-    (-when-let (process-buf (process-buffer process))
+    (when-let ((process-buf (process-buffer process)))
       (when (buffer-live-p process-buf)
-        (-when-let (status-buf (with-current-buffer process-buf
-                                 (magit-mode-get-buffer 'magit-status-mode)))
+        (when-let ((status-buf (with-current-buffer process-buf
+                                 (magit-mode-get-buffer 'magit-status-mode))))
           (with-current-buffer status-buf
             (--when-let
                 (magit-get-section
@@ -715,7 +720,7 @@ Magit status buffer."
 
 (defun magit-process-yes-or-no-prompt (process string)
   "Forward Yes-or-No prompts to the user."
-  (-when-let (beg (string-match magit-process-yes-or-no-prompt-regexp string))
+  (when-let ((beg (string-match magit-process-yes-or-no-prompt-regexp string)))
     (let ((max-mini-window-height 30))
       (process-send-string
        process
@@ -730,14 +735,32 @@ Magit status buffer."
 
 (defun magit-process-password-auth-source (key)
   "Use `auth-source-search' to get a password.
-If found, return the password.  Otherwise, return nil."
+If found, return the password.  Otherwise, return nil.
+
+To use this function add it to the appropriate hook
+  (add-hook 'magit-process-find-password-functions
+            'magit-process-password-auth-source)
+
+KEY typically derives from a prompt such as:
+  Password for 'https://tarsius@bitbucket.org'
+in which case it would be the string
+  tarsius@bitbucket.org
+which matches the ~/.authinfo.gpg entry
+  machine bitbucket.org login tarsius password 12345
+or iff that is undefined, for backward compatibility
+  machine tarsius@bitbucket.org password 12345"
   (require 'auth-source)
-  (let ((secret (plist-get (car (auth-source-search :max 1 :host key
-                                                    :require '(:host)))
-                           :secret)))
-    (if (functionp secret)
-        (funcall secret)
-      secret)))
+  (and (string-match "\\`\\([^@]+\\)@\\([^@]+\\)\\'" key)
+       (let* ((user (match-string 1 key))
+              (host (match-string 2 key))
+              (secret
+               (plist-get
+                (car (or (auth-source-search :max 1 :host host :user user)
+                         (auth-source-search :max 1 :host key)))
+                :secret)))
+         (if (functionp secret)
+             (funcall secret)
+           secret))))
 
 (defun magit-process-password-prompt (process string)
   "Find a password based on prompt STRING and send it to git.
@@ -746,14 +769,14 @@ prompt.  If and only if one is found, then call functions in
 `magit-process-find-password-functions' until one of them returns
 the password.  If all function return nil, then read the password
 from the user."
-  (--when-let (magit-process-match-prompt
-               magit-process-password-prompt-regexps string)
+  (when-let ((prompt (magit-process-match-prompt
+                      magit-process-password-prompt-regexps string)))
     (process-send-string
      process (magit-process-kill-on-abort process
-               (concat (or (--when-let (match-string 99 string)
+               (concat (or (when-let ((key (match-string 99 string)))
                              (run-hook-with-args-until-success
-                              'magit-process-find-password-functions it))
-                           (read-passwd it))
+                              'magit-process-find-password-functions key))
+                           (read-passwd prompt))
                        "\n")))))
 
 (defun magit-process-username-prompt (process string)
@@ -799,8 +822,9 @@ as argument."
               (memq magit-credential-cache-daemon-process
                     (list-system-processes)))
     (setq magit-credential-cache-daemon-process
-          (or (--first (-let (((&alist 'comm comm 'user user)
-                               (process-attributes it)))
+          (or (--first (let* ((attr (process-attributes it))
+                              (comm (cdr (assq 'comm attr)))
+                              (user (cdr (assq 'user attr))))
                          (and (string= comm "git-credential-cache--daemon")
                               (string= user user-login-name)))
                        (list-system-processes))
@@ -853,7 +877,8 @@ as argument."
   (when (equal program magit-git-executable)
     (setq args (nthcdr (length magit-git-global-arguments) args)))
   (let ((str (concat " " (propertize
-                          (concat program (and args (concat " " (car args))))
+                          (concat (file-name-nondirectory program)
+                                  (and args (concat " " (car args))))
                           'mouse-face 'highlight
                           'keymap magit-mode-line-process-map
                           'help-echo "mouse-1: Show process buffer"
@@ -1040,14 +1065,14 @@ Limited by `magit-process-error-tooltip-max-lines'."
        ((not (eq msg 'suppressed))
         (when (buffer-live-p process-buf)
           (with-current-buffer process-buf
-            (-when-let (status-buf (magit-mode-get-buffer 'magit-status-mode))
+            (when-let ((status-buf (magit-mode-get-buffer 'magit-status-mode)))
               (with-current-buffer status-buf
                 (setq magit-this-error msg)))))
         (message "%s ... [%s buffer %s for details]" msg
-                 (-if-let (key (and (buffer-live-p command-buf)
+                 (if-let ((key (and (buffer-live-p command-buf)
                                     (with-current-buffer command-buf
                                       (car (where-is-internal
-                                            'magit-process-buffer)))))
+                                            'magit-process-buffer))))))
                      (format "Hit %s to see" (key-description key))
                    "See")
                  (buffer-name process-buf))))))
@@ -1092,5 +1117,6 @@ Limited by `magit-process-error-tooltip-max-lines'."
           (setq lines (cons heading lines)))
         (message (mapconcat #'identity lines "\n"))))))
 
+;;; _
 (provide 'magit-process)
 ;;; magit-process.el ends here
