@@ -33,6 +33,9 @@
 (require 'dash)
 (require 'eieio)
 
+(eval-when-compile
+  (require 'subr-x))
+
 (require 'magit-utils)
 
 (declare-function magit-maybe-make-margin-overlay "magit-margin" ())
@@ -129,7 +132,8 @@ then don't do so for any sections."
                  (const  :tag "Cache visibility of all sections" t)
                  (repeat :tag "Cache visibility for section types" symbol)))
 
-(defcustom magit-section-initial-visibility-alist nil
+(defcustom magit-section-initial-visibility-alist
+  '((stashes . hide))
   "Alist controlling the initial visibility of sections.
 
 Each element maps a section type or lineage to the initial
@@ -143,7 +147,12 @@ and the type is the first element of that vector.  Wildcards can
 be used, see `magit-section-match'.
 
 Currently this option is only used to override hardcoded defaults,
-but in the future it will also be used set the defaults."
+but in the future it will also be used set the defaults.
+
+An entry whose key is `magit-status-initial-section' specifies
+the visibility of the section `magit-status-goto-initial-section'
+jumps to.  This does not only override defaults, but also other
+entries of this alist."
   :package-version '(magit . "2.12.0")
   :group 'magit-section
   :type '(alist :key-type (sexp :tag "Section type/lineage")
@@ -173,19 +182,31 @@ but in the future it will also be used set the defaults."
   "Face for selected section headings."
   :group 'magit-faces)
 
-;;; Core
+;;; Classes
+
+(defvar magit--current-section-hook nil
+  "Internal variable used for `magit-explain-section'.")
+
+(defvar magit--section-type-alist
+  '(
+    (file            . magit-file-section)
+    (hunk            . magit-hunk-section)
+    (module          . magit-module-section)
+    ))
 
 (defclass magit-section ()
-  ((type     :initform nil :accessor magit-section-type     :initarg :type)
-   (value    :initform nil :accessor magit-section-value    :initarg :value)
-   (start    :initform nil :accessor magit-section-start    :initarg :start)
-   (content  :initform nil :accessor magit-section-content)
-   (end      :initform nil :accessor magit-section-end)
-   (hidden   :initform nil :accessor magit-section-hidden)
-   (washer   :initform nil :accessor magit-section-washer)
+  ((type     :initform nil :initarg :type)
+   (value    :initform nil :initarg :value)
+   (start    :initform nil :initarg :start)
+   (content  :initform nil)
+   (end      :initform nil)
+   (hidden   :initform nil)
+   (washer   :initform nil)
    (process  :initform nil)
-   (parent   :initform nil :accessor magit-section-parent   :initarg :parent)
-   (children :initform nil :accessor magit-section-children)))
+   (heading-highlight-face :initform nil)
+   (inserter :initform (symbol-value 'magit--current-section-hook))
+   (parent   :initform nil :initarg :parent)
+   (children :initform nil)))
 
 (defclass magit-file-section (magit-section)
   ((source   :initform nil)
@@ -194,6 +215,11 @@ but in the future it will also be used set the defaults."
 (defclass magit-hunk-section (magit-section)
   ((refined  :initform nil)))
 
+(defclass magit-module-section (magit-file-section)
+  ())
+
+;;; Core
+
 (defvar-local magit-root-section nil
   "The root section in the current buffer.
 All other sections are descendants of this section.  The value
@@ -242,8 +268,10 @@ IDENT has to be a list as returned by `magit-section-ident'."
 
 (defun magit-section-lineage (section)
   "Return the lineage of SECTION.
-The return value has the form [TYPE...]."
-  (apply #'vector (mapcar #'car (magit-section-ident section))))
+The return value has the form (TYPE...)."
+  (cons (oref section type)
+        (when-let ((parent (oref section parent)))
+          (magit-section-lineage parent))))
 
 (defvar magit-insert-section--current nil "For internal use only.")
 (defvar magit-insert-section--parent  nil "For internal use only.")
@@ -380,7 +408,7 @@ With a prefix argument also expand it." heading)
   "Show the body of the current section."
   (interactive (list (magit-current-section)))
   (oset section hidden nil)
-  (-when-let (washer (oref section washer))
+  (when-let ((washer (oref section washer)))
     (oset section washer nil)
     (let ((inhibit-read-only t)
           (magit-insert-section--parent section)
@@ -393,7 +421,7 @@ With a prefix argument also expand it." heading)
           (funcall washer)
           (oset section end (point-marker)))))
     (magit-section-update-highlight))
-  (-when-let (beg (oref section content))
+  (when-let ((beg (oref section content)))
     (remove-overlays beg (oref section end) 'invisible t))
   (magit-section-maybe-cache-visibility section)
   (dolist (child (oref section children))
@@ -407,7 +435,7 @@ With a prefix argument also expand it." heading)
   (if (eq section magit-root-section)
       (user-error "Cannot hide root section")
     (oset section hidden t)
-    (-when-let (beg (oref section content))
+    (when-let ((beg (oref section content)))
       (let ((end (oref section end)))
         (remove-overlays beg end 'invisible t)
         (let ((o (make-overlay beg end)))
@@ -444,13 +472,13 @@ children."
   (magit-section-show section))
 
 (defun magit-section-show-children-1 (section &optional depth)
-  (--each (oref section children)
-    (oset it hidden nil)
+  (dolist (child (oref section children))
+    (oset child hidden nil)
     (if depth
         (if (> depth 0)
-            (magit-section-show-children-1 it (1- depth))
-          (magit-section-hide it))
-      (magit-section-show-children-1 it))))
+            (magit-section-show-children-1 child (1- depth))
+          (magit-section-hide child))
+      (magit-section-show-children-1 child))))
 
 (defun magit-section-hide-children (section)
   "Recursively hide the bodies of children of the current section."
@@ -466,11 +494,11 @@ hidden."
   (magit-section-show section))
 
 (defun magit-section-show-headings-1 (section)
-  (--each (oref section children)
-    (oset it hidden nil)
-    (when (or (oref it children)
-              (not (oref it content)))
-      (magit-section-show-headings-1 it))))
+  (dolist (child (oref section children))
+    (oset child hidden nil)
+    (when (or (oref child children)
+              (not (oref child content)))
+      (magit-section-show-headings-1 child))))
 
 (defun magit-section-cycle (section)
   "Cycle visibility of current section and its children."
@@ -503,7 +531,7 @@ hidden."
 (defun magit-section-cycle-diffs ()
   "Cycle visibility of diff-related sections in the current buffer."
   (interactive)
-  (-when-let (sections
+  (when-let ((sections
               (cond ((derived-mode-p 'magit-status-mode)
                      (--mapcat
                       (when it
@@ -514,7 +542,7 @@ hidden."
                             (magit-get-section '((unstaged) (status))))))
                     ((derived-mode-p 'magit-diff-mode)
                      (-filter #'magit-file-section-p
-                              (oref magit-root-section children)))))
+                              (oref magit-root-section children))))))
     (if (--any-p (oref it hidden) sections)
         (dolist (s sections)
           (magit-section-show s)
@@ -603,16 +631,75 @@ Sections at higher levels are hidden."
 
 ;;;; Auxiliary
 
-(defun magit-describe-section ()
+(defun magit-describe-section-briefly (section &optional message ident)
   "Show information about the section at point.
-This command is intended for debugging purposes."
-  (interactive)
-  (let ((section (magit-current-section)))
-    (message "%S %S %s-%s"
-             (oref section value)
-             (magit-section-lineage section)
-             (marker-position (oref section start))
-             (marker-position (oref section end)))))
+With a prefix argument show the section identity instead of the
+section lineage.  This command is intended for debugging purposes."
+  (interactive (list (magit-current-section) t))
+  (let ((str (format "#<%s %S %S %s-%s>"
+                     (eieio-object-class section)
+                     (let ((val (oref section value)))
+                       (cond ((stringp val)
+                              (substring-no-properties val))
+                             ((and (eieio-object-p val)
+                                   (fboundp 'cl-prin1-to-string))
+                              (cl-prin1-to-string val))
+                             (t
+                              val)))
+                     (if ident
+                         (magit-section-ident section)
+                       (apply #'vector (magit-section-lineage section)))
+                     (when-let ((m (oref section start)))
+                       (marker-position m))
+                     (when-let ((m (oref section end)))
+                       (marker-position m)))))
+    (if message (message "%s" str) str)))
+
+(cl-defmethod cl-print-object ((section magit-section) stream)
+  "Print `magit-describe-section' result of SECTION."
+  ;; Used by debug and edebug as of Emacs 26.
+  (princ (magit-describe-section-briefly section) stream))
+
+(defun magit-describe-section (section &optional interactive-p)
+  "Show information about the section at point."
+  (interactive (list (magit-current-section) t))
+  (let ((inserter-section section))
+    (while (and inserter-section (not (oref inserter-section inserter)))
+      (setq inserter-section (oref inserter-section parent)))
+    (when (and inserter-section (oref inserter-section inserter))
+      (setq section inserter-section)))
+  (pcase (oref section inserter)
+    (`((,hook ,fun) . ,src-src)
+     (help-setup-xref `(magit-describe-section ,section) interactive-p)
+     (with-help-window (help-buffer)
+       (with-current-buffer standard-output
+         (insert (format-message
+                  "%s\n  is inserted by `%s'\n  from `%s'"
+                  (magit-describe-section-briefly section)
+                  (make-text-button (symbol-name fun) nil
+                                    :type 'help-function
+                                    'help-args (list fun))
+                  (make-text-button (symbol-name hook) nil
+                                    :type 'help-variable
+                                    'help-args (list hook))))
+         (pcase-dolist (`(,hook ,fun) src-src)
+           (insert (format-message
+                    ",\n  called by `%s'\n  from `%s'"
+                    (make-text-button (symbol-name fun) nil
+                                      :type 'help-function
+                                      'help-args (list fun))
+                    (make-text-button (symbol-name hook) nil
+                                      :type 'help-variable
+                                      'help-args (list hook)))))
+         (insert ".\n\n")
+         (insert
+          (format-message
+           "`%s' is "
+           (make-text-button (symbol-name fun) nil
+                             :type 'help-function 'help-args (list fun))))
+         (describe-function-1 fun))))
+    (_ (message "%s, inserter unknown"
+                (magit-describe-section-briefly section)))))
 
 ;;; Match
 
@@ -626,39 +713,66 @@ nil.
 
 CONDITION can take the following forms:
   (CONDITION...)  matches if any of the CONDITIONs matches.
-  [TYPE...]       matches if the first TYPE matches the type
-                  of the section, the second matches that of
-                  its parent, and so on.
-  [* TYPE...]     matches sections that match [TYPE...] and
+  [CLASS...]      matches if the section's class is the same
+                  as the first CLASS or a subclass of that;
+                  the section's parent class matches the
+                  second CLASS; and so on.
+  [* CLASS...]    matches sections that match [CLASS...] and
                   also recursively all their child sections.
-  TYPE            matches sections of TYPE regardless of the
-                  types of the parent sections.
+  CLASS           matches if the section's class is the same
+                  as CLASS or a subclass of that; regardless
+                  of the classes of the parent sections.
 
-Each TYPE is a symbol.  Note that it is not necessary to specify
-all TYPEs up to the root section as printed by
-`magit-describe-type', unless of course you want to be that
-precise."
+Each CLASS should be a class symbol, identifying a class that
+derives from `magit-section'.  For backward compatibility CLASS
+can also be a \"type symbol\".  A section matches such a symbol
+if the value of its `type' slot is `eq'.  If a type symbol has
+an entry in `magit--section-type-alist', then a section also
+matches that type if its class is a subclass of the class that
+corresponds to the type as per that alist.
+
+Note that it is not necessary to specify the complete section
+lineage as printed by `magit-describe-section-briefly', unless
+of course you want to be that precise."
+  (and section (magit-section-match-1 condition section)))
+
+(defun magit-section-match-1 (condition section)
+  (cl-assert condition)
   (and section
-       (magit-section-match-1 condition
-                              (mapcar #'car (magit-section-ident section)))))
+       (if (listp condition)
+           (--first (magit-section-match-1 it section) condition)
+         (magit-section-match-2 (if (symbolp condition)
+                                    (list condition)
+                                  (cl-coerce condition 'list))
+                                section))))
 
-(defun magit-section-match-1 (condition type-list)
-  (if (listp condition)
-      (--first (magit-section-match-1 it type-list) condition)
-    (magit-section-match-2 (if (symbolp condition)
-                               (list condition)
-                             (append condition nil))
-                           type-list)))
+(defun magit-section-match-2 (condition section)
+  (if (eq (car condition) '*)
+      (or (magit-section-match-2 (cdr condition) section)
+          (when-let ((parent (oref section parent)))
+            (magit-section-match-2 condition parent)))
+    (and (let ((c (car condition)))
+           (if (class-p c)
+               (cl-typep section c)
+             (if-let ((class (cdr (assq c magit--section-type-alist))))
+                 (cl-typep section class)
+               (eq (oref section type) c))))
+         (or (not (setq condition (cdr condition)))
+             (when-let ((parent (oref section parent)))
+               (magit-section-match-2 condition parent))))))
 
-(defun magit-section-match-2 (l1 l2)
-  (or (null l1)
-      (if (eq (car l1) '*)
-          (or (magit-section-match-2 (cdr l1) l2)
-              (and l2
-                   (magit-section-match-2 l1 (cdr l2))))
-        (and l2
-             (equal (car l1) (car l2))
-             (magit-section-match-2 (cdr l1) (cdr l2))))))
+(defun magit-section-value-if (condition &optional section)
+  "If the section at point matches CONDITION, then return its value.
+
+If optional SECTION is non-nil then test whether that matches
+instead.  If there is no section at point and SECTION is nil,
+then return nil.  If the section does not match, then return
+nil.
+
+See `magit-section-match' for the forms CONDITION can take."
+  (when-let ((section (or section (magit-current-section))))
+    (and (magit-section-match condition section)
+         (oref section value))))
 
 (defmacro magit-section-when (condition &rest body)
   "If the section at point matches CONDITION, evaluate BODY.
@@ -670,7 +784,10 @@ section.  If the section does not match or if there is no section
 at point, then return nil.
 
 See `magit-section-match' for the forms CONDITION can take."
-  (declare (indent 1)
+  (declare (obsolete
+            "instead use `magit-section-match' or `magit-section-value-if'."
+            "Magit 2.90.0")
+           (indent 1)
            (debug (sexp body)))
   `(--when-let (magit-current-section)
      ;; Quoting CONDITION here often leads to double-quotes, which
@@ -697,22 +814,19 @@ matches if no other CONDITION match, even if there is no section
 at point."
   (declare (indent 0)
            (debug (&rest (sexp body))))
-  (let ((lineage (cl-gensym "lineage")))
-    `(let* ((it (magit-current-section))
-            (,lineage (and it (append (magit-section-lineage it) nil))))
-       (cond ,@(mapcar (lambda (clause)
-                         `(,(or (eq (car clause) t)
-                                `(and it (magit-section-match-1
-                                          ',(car clause) ,lineage)))
-                           ,@(cdr clause)))
-                       clauses)))))
+  `(let* ((it (magit-current-section)))
+     (cond ,@(mapcar (lambda (clause)
+                       `(,(or (eq (car clause) t)
+                              `(and it
+                                    (magit-section-match-1 ',(car clause) it)))
+                         ,@(cdr clause)))
+                     clauses))))
 
 (defun magit-section-match-assoc (section alist)
   "Return the value associated with SECTION's type or lineage in ALIST."
-  (let ((ident (mapcar #'car (magit-section-ident section))))
-    (--some (pcase-let ((`(,key . ,val) it))
-              (and (magit-section-match-1 key ident) val))
-            alist)))
+  (-some (pcase-lambda (`(,key . ,val))
+           (and (magit-section-match-1 key section) val))
+         alist))
 
 ;;; Create
 
@@ -773,26 +887,30 @@ anything this time around.
                     (if (eq (car-safe type) 'eval)
                         (cadr type)
                       `',type)))
-            (,s (funcall (pcase ,tp
-                           (`file 'magit-file-section)
-                           (`hunk 'magit-hunk-section)
-                           (_     'magit-section))
-                         ""
-                         :type ,tp
+            (,s (funcall (if (class-p ,tp)
+                             ,tp
+                           (or (cdr (assq ,tp magit--section-type-alist))
+                               'magit-section))
+                         :type
+                         (if (class-p ,tp)
+                             (or (car (rassq ,tp magit--section-type-alist))
+                                 (error "BUG: No entry for %s in %s" ,tp
+                                        'magit--section-type-alist))
+                           ,tp)
                          :value ,(nth 1 (car args))
                          :start (point-marker)
                          :parent magit-insert-section--parent)))
        (oset ,s hidden
-             (-if-let (value (run-hook-with-args-until-success
-                              'magit-section-set-visibility-hook ,s))
+             (if-let ((value (run-hook-with-args-until-success
+                              'magit-section-set-visibility-hook ,s)))
                  (eq value 'hide)
-               (-if-let (incarnation (and magit-insert-section--oldroot
+               (if-let ((incarnation (and magit-insert-section--oldroot
                                           (magit-get-section
                                            (magit-section-ident ,s)
-                                           magit-insert-section--oldroot)))
+                                           magit-insert-section--oldroot))))
                    (oref incarnation hidden)
-                 (-if-let (value (magit-section-match-assoc
-                                  ,s magit-section-initial-visibility-alist))
+                 (if-let ((value (magit-section-match-assoc
+                                  ,s magit-section-initial-visibility-alist)))
                      (progn
                        (when (functionp value)
                          (setq value (funcall value ,s)))
@@ -810,12 +928,19 @@ anything this time around.
                  `((let ((,s* ,s))
                      ,@(cdr args)))
                (cdr args))
+           ;; `magit-insert-section-hook' should *not* be run with
+           ;; `magit-run-section-hook' because it's a hook that runs
+           ;; on section insertion, not a section inserting hook.
            (run-hooks 'magit-insert-section-hook)
            (magit-insert-child-count ,s)
            (set-marker-insertion-type (oref ,s start) t)
            (let* ((end (oset ,s end (point-marker)))
-                  (map (intern (format "magit-%s-section-map" (oref ,s type))))
-                  (map (and (boundp map) (symbol-value map))))
+                  (magit-map (intern (format "magit-%s-section-map"
+                                             (oref ,s type))))
+                  (forge-map (intern (format "forge-%s-section-map"
+                                             (oref ,s type))))
+                  (map (or (and (boundp magit-map) (symbol-value magit-map))
+                           (and (boundp forge-map) (symbol-value forge-map)))))
              (save-excursion
                (goto-char (oref ,s start))
                (while (< (point) end)
@@ -882,29 +1007,29 @@ insert a newline character if necessary."
   (magit-maybe-make-margin-overlay)
   (oset magit-insert-section--current content (point-marker)))
 
-(defvar magit-insert-headers--hook nil "For internal use only.")
-(defvar magit-insert-headers--beginning nil "For internal use only.")
-
-(defun magit-insert-headers (hooks)
-  (let ((magit-insert-section-hook
-         (cons 'magit-insert-remaining-headers
-               (if (listp magit-insert-section-hook)
-                   magit-insert-section-hook
-                 (list magit-insert-section-hook))))
-        (magit-insert-headers--hook hooks)
-        wrapper)
-    (setq magit-insert-headers--beginning (point))
-    (while (and (setq wrapper (pop magit-insert-headers--hook))
-                (= (point) magit-insert-headers--beginning))
-      (funcall wrapper))))
-
-(defun magit-insert-remaining-headers ()
-  (if (= (point) magit-insert-headers--beginning)
-      (magit-cancel-section)
-    (magit-insert-heading)
-    (remove-hook 'magit-insert-section-hook 'magit-insert-remaining-headers)
-    (mapc #'funcall magit-insert-headers--hook)
-    (insert "\n")))
+(defun magit-insert-headers (hook)
+  (let* ((header-sections nil)
+         (magit-insert-section-hook
+          (cons (lambda ()
+                  (push magit-insert-section--current
+                        header-sections))
+                (if (listp magit-insert-section-hook)
+                    magit-insert-section-hook
+                  (list magit-insert-section-hook)))))
+    (magit-run-section-hook hook)
+    (when header-sections
+      (insert "\n")
+      ;; Make the first header into the parent of the rest.
+      (when (cdr header-sections)
+        (cl-callf nreverse header-sections)
+        (let* ((1st-header (pop header-sections))
+               (header-parent (oref 1st-header parent)))
+          (oset header-parent children (list 1st-header))
+          (oset 1st-header children header-sections)
+          (oset 1st-header content (oref (car header-sections) start))
+          (oset 1st-header end (oref (car (last header-sections)) end))
+          (dolist (sub-header header-sections)
+            (oset sub-header parent 1st-header)))))))
 
 (defun magit-insert-child-count (section)
   "Modify SECTION's heading to contain number of child sections.
@@ -956,9 +1081,9 @@ evaluated its BODY.  Admittedly that's a bit of a hack."
         (unless (eq section magit-root-section)
           (run-hook-with-args-until-success
            'magit-section-highlight-hook section selection))
-        (--each magit-section-unhighlight-sections
+        (dolist (s magit-section-unhighlight-sections)
           (run-hook-with-args-until-success
-           'magit-section-unhighlight-hook it selection))
+           'magit-section-unhighlight-hook s selection))
         (restore-buffer-modified-p nil)
         (unless (eq magit-section-highlighted-section section)
           (setq magit-section-highlighted-section
@@ -972,6 +1097,13 @@ evaluated its BODY.  Admittedly that's a bit of a hack."
 This function works for any section but produces undesirable
 effects for diff related sections, which by default are
 highlighted using `magit-diff-highlight'.  Return t."
+  (when-let ((face (oref section heading-highlight-face)))
+    (dolist (section (or selection (list section)))
+      (magit-section-make-overlay
+       (oref section start)
+       (or (oref section content)
+           (oref section end))
+       face)))
   (cond (selection
          (magit-section-make-overlay (oref (car selection) start)
                                      (oref (car (last selection)) end)
@@ -998,10 +1130,10 @@ part of the hook variable, then such a region would be
 invisible."
   (when (and selection
              (not (and (eq this-command 'mouse-drag-region))))
-    (--each selection
-      (magit-section-make-overlay (oref it start)
-                                  (or (oref it content)
-                                      (oref it end))
+    (dolist (section selection)
+      (magit-section-make-overlay (oref section start)
+                                  (or (oref section content)
+                                      (oref section end))
                                   'magit-section-heading-selection))
     t))
 
@@ -1031,9 +1163,9 @@ invisible."
             (unless (eq (magit-current-section) it)
               (goto-char start))))
       (or (and (magit-hunk-section-p section)
-               (-when-let (parent (magit-get-section
+               (when-let ((parent (magit-get-section
                                    (magit-section-ident
-                                    (oref section parent))))
+                                    (oref section parent)))))
                  (let* ((children (oref parent children))
                         (siblings (magit-section-siblings section 'prev))
                         (previous (nth (length siblings) children)))
@@ -1084,7 +1216,7 @@ invisible."
 
 (cl-defun magit-section-cache-visibility
     (&optional (section magit-insert-section--current))
-  ;; Emacs 24 doesn't have `alist-get'.
+  ;; Emacs 25's `alist-get' lacks TESTFN.
   (let* ((id  (magit-section-ident section))
          (elt (assoc id magit-section-visibility-cache))
          (val (if (oref section hidden) 'hide 'show)))
@@ -1122,7 +1254,7 @@ invisible."
               (magit-section-selected-p it selection)))))
 
 (defun magit-section-parent-value (section)
-  (-when-let (parent (oref section parent))
+  (when-let ((parent (oref section parent)))
     (oref parent value)))
 
 (defun magit-section-siblings (section &optional direction)
@@ -1132,26 +1264,22 @@ If optional DIRECTION is `prev', then return siblings that come
 before SECTION.  If it is `next', then return siblings that come
 after SECTION.  For all other values, return all siblings
 excluding SECTION itself."
-  (-when-let (parent (oref section parent))
+  (when-let ((parent (oref section parent)))
     (let ((siblings (oref parent children)))
       (pcase direction
         (`prev  (cdr (member section (reverse siblings))))
         (`next  (cdr (member section siblings)))
         (_      (remq section siblings))))))
 
-(defun magit-region-values (&optional types multiple)
+(defun magit-region-values (&optional condition multiple)
   "Return a list of the values of the selected sections.
 
-Also see `magit-region-sections' whose doc-string explains when a
-region is a valid section selection.  If the region is not active
-or is not a valid section selection, then return nil.  If optional
-TYPES is non-nil then the selection not only has to be valid; the
-types of all selected sections additionally have to match one of
-TYPES, or nil is returned."
+Return the values that themselves would be returned by
+`magit-region-sections' (which see)."
   (--map (oref it value)
-         (magit-region-sections types multiple)))
+         (magit-region-sections condition multiple)))
 
-(defun magit-region-sections (&optional types multiple)
+(defun magit-region-sections (&optional condition multiple)
   "Return a list of the selected sections.
 
 When the region is active and constitutes a valid section
@@ -1172,10 +1300,10 @@ apply to diffs where things get a bit more complicated, but even
 here if the region looks like it usually does, then that's not
 a valid selection as far as this function is concerned.
 
-If optional TYPES is non-nil, then the selection not only has to
-be valid; the types of all selected sections additionally have
-to match one of TYPES, or nil is returned.  TYPES can also be a
-single type, instead of a list of types."
+If optional CONDITION is non-nil, then the selection not only
+has to be valid; all selected sections additionally have to match
+CONDITION, or nil is returned.  See `magit-section-match' for the
+forms CONDITION can take."
   (when (region-active-p)
     (let* ((rbeg (region-beginning))
            (rend (region-end))
@@ -1194,10 +1322,8 @@ single type, instead of a list of types."
               (when (eq (pop siblings) send)
                 (setq siblings nil)))
             (setq sections (nreverse sections))
-            (when (and types (symbolp types))
-              (setq types (list types)))
-            (when (or (not types)
-                      (--all-p (memq (oref it type) types) sections))
+            (when (or (not condition)
+                      (--all-p (magit-section-match condition it) sections))
               sections)))))))
 
 (defun magit-section-position-in-heading-p (&optional section pos)
@@ -1309,20 +1435,23 @@ again use `remove-hook'."
         (set hook value)
       (set-default hook value))))
 
-(defun magit-run-section-hook (hook)
-  "Run HOOK, warning about invalid entries."
-  (--if-let (-remove #'functionp (symbol-value hook))
-      (progn
-        (message "`%s' contains entries that are no longer valid.
+(defun magit-run-section-hook (hook &rest args)
+  "Run HOOK with ARGS, warning about invalid entries."
+  (let ((entries (symbol-value hook)))
+    (unless (listp entries)
+      (setq entries (list entries)))
+    (--when-let (-remove #'functionp entries)
+      (message "`%s' contains entries that are no longer valid.
 %s\nUsing standard value instead.  Please re-configure hook variable."
-                 hook
-                 (mapconcat (lambda (sym) (format "  `%s'" sym)) it "\n"))
-        (sit-for 5)
-        (defvar magit--hook-standard-value nil)
-        (let ((magit--hook-standard-value
-               (eval (car (get hook 'standard-value)))))
-          (run-hooks 'magit---hook-standard-value)))
-    (run-hooks hook)))
+               hook
+               (mapconcat (lambda (sym) (format "  `%s'" sym)) it "\n"))
+      (sit-for 5)
+      (setq entries (eval (car (get hook 'standard-value)))))
+    (dolist (entry entries)
+      (let ((magit--current-section-hook (cons (list hook entry)
+                                               magit--current-section-hook)))
+        (apply entry args)))))
 
+;;; _
 (provide 'magit-section)
 ;;; magit-section.el ends here
