@@ -19,7 +19,7 @@
 ;; Keywords: git tools vc
 ;; Homepage: https://github.com/magit/magit
 
-;; Magit requires at least GNU Emacs 24.4 and Git 1.9.4.
+;; Magit requires at least GNU Emacs 25.1 and Git 2.0.0.
 
 ;; Magit is free software; you can redistribute it and/or modify it
 ;; under the terms of the GNU General Public License as published by
@@ -50,6 +50,9 @@
 (require 'cl-lib)
 (require 'dash)
 
+(eval-when-compile
+  (require 'subr-x))
+
 (require 'with-editor)
 (require 'git-commit)
 (require 'magit-core)
@@ -61,8 +64,8 @@
 (require 'format-spec)
 (require 'package nil t) ; used in `magit-version'
 
-(defconst magit--minimal-git "1.9.4")
-(defconst magit--minimal-emacs "24.4")
+(defconst magit--minimal-git "2.0.0")
+(defconst magit--minimal-emacs "25.1")
 
 ;;; Faces
 
@@ -121,6 +124,14 @@ own faces for the `header-line', or for parts of the
   "Face for current branch."
   :group 'magit-faces)
 
+(defface magit-branch-upstream
+  '((t :slant italic))
+  "Face for upstream branch.
+This face is only used in logs and it gets combined
+ with `magit-branch-local', `magit-branch-remote'
+and/or `magit-branch-remote-head'."
+  :group 'magit-faces)
+
 (defface magit-head
   '((((class color) (background light)) :inherit magit-branch-local)
     (((class color) (background  dark)) :inherit magit-branch-local))
@@ -135,7 +146,7 @@ own faces for the `header-line', or for parts of the
 
 (defface magit-refname-stash
   '((t :inherit magit-refname))
-  "Face for wip refnames."
+  "Face for stash refnames."
   :group 'magit-faces)
 
 (defface magit-refname-wip
@@ -143,11 +154,21 @@ own faces for the `header-line', or for parts of the
   "Face for wip refnames."
   :group 'magit-faces)
 
+(defface magit-refname-pullreq
+  '((t :inherit magit-refname))
+  "Face for pullreq refnames."
+  :group 'magit-faces)
+
 (defface magit-keyword
   '((t :inherit font-lock-string-face))
   "Face for parts of commit messages inside brackets."
   :group 'magit-faces)
 
+(defface magit-keyword-squash
+  '((t :inherit font-lock-warning-face))
+  "Face for squash! and fixup! keywords in commit messages."
+  :group 'magit-faces)
+
 (defface magit-signature-good
   '((t :foreground "green"))
   "Face for good signatures."
@@ -385,7 +406,6 @@ is run in the top-level directory of the current working tree."
        (2 'font-lock-function-name-face nil t))
       (,(concat "(" (regexp-opt '("magit-insert-section"
                                   "magit-section-case"
-                                  "magit-section-when"
                                   "magit-bind-match-strings"
                                   "magit-with-temp-index"
                                   "magit-with-blob"
@@ -454,7 +474,7 @@ and Emacs to it."
                              (package-version-join
                               (package-desc-version (cadr it))))))))
             (progn
-              (push 'debug debug)
+              (push 'dirname debug)
               (let ((dirname (file-name-nondirectory
                               (directory-file-name topdir))))
                 (when (string-match "\\`magit-\\([0-9]\\{8\\}\\.[0-9]*\\)"
@@ -521,6 +541,14 @@ See info node `(magit)Debugging Tools' for more information."
 ;;; Startup Asserts
 
 (defun magit-startup-asserts ()
+  (when-let ((val (getenv "GIT_DIR")))
+    (setenv "GIT_DIR")
+    (message "Magit unset $GIT_DIR (was %S).  See \
+https://github.com/magit/magit/wiki/Don't-set-$GIT_DIR-and-alike" val))
+  (when-let ((val (getenv "GIT_WORK_TREE")))
+    (setenv "GIT_WORK_TREE")
+    (message "Magit unset $GIT_WORK_TREE (was %S).  See \
+https://github.com/magit/magit/wiki/Don't-set-$GIT_DIR-and-alike" val))
   (let ((version (magit-git-version)))
     (when (and version
                (version< version magit--minimal-git)
@@ -573,12 +601,16 @@ For X11 something like ~/.xinitrc should work.\n"
   (require 'magit-sequence)
   (require 'magit-commit)
   (require 'magit-remote)
+  (require 'magit-clone)
+  (require 'magit-fetch)
+  (require 'magit-pull)
+  (require 'magit-push)
   (require 'magit-bisect)
   (require 'magit-stash)
   (require 'magit-blame)
   (require 'magit-obsolete)
+  (require 'magit-submodule)
   (unless (load "magit-autoloads" t t)
-    (require 'magit-submodule)
     (require 'magit-subtree)
     (require 'magit-ediff)
     (require 'magit-extras)
